classdef SystHBQ < Syst
    
    properties
        
        H;  % Order of truncation of Fourier series
        
        subtype = 'autonomous' ; % subtype of system: {['autonomous'],'forced'}
        
        bif_type = 'nothing';    % bifurcation following : 'NS', 'PD' , 'SB' (Neimarck-Sacker, Period Doubling, Simple Bifurcation)
        eq_old;                  % equations of the original system in the case of bifurcation following.
        
        nz_tot ;    % number of equation of the DAE
        nz ;        % number of main equations
        nz_aux=0;   % number of auxiliary equations
        
        angfreq='omega';  % for forced system, the continuation parameter is omega by default. 
                        % Otherwise, sys.angfreq is the value of the
                        % angfreq of the forcing terms
        
        % Small operator on the Algebro-differential equations
        id =[]; jd =[]; vd= [];                % list for the sparse (order 2) tensor d
        id1=[]; jd1=[]; vd1=[];                % list for the sparse (order 2) tensor d1
        idd=[]; jdd=[]; vdd=[];                % list for the sparse (order 2) tensor dd
        ic0=[]; vc0=[];                        % list for the sparse (order 1) tensor c0
        ic1=[]; vc1=[];                        % list for the sparse (order 1) tensor c1
        ic2=[]; vc2=[];                        % list for the sparse (order 1) tensor c1
        il0=[]; jl0=[]; vl0=[];                % list for the sparse (order 2) tensor l0
        il1=[]; jl1=[]; vl1=[];                % list for the sparse (order 2) tensor l1
        iq =[]; jq =[]; kq =[]; vq=[];         % list for the sparse (order 3) tensor q
        uiq=[];i_iq=[];

        iforce0=[]; hforce0=[]; vforce0=[];       % list for the sparse (order 2) tensor force
        iforce1=[]; hforce1=[]; vforce1=[];       % list for the sparse (order 2) tensor force
        iforce2=[]; hforce2=[]; vforce2=[];       % list for the sparse (order 2) tensor force

        idl=[]; jdl=[]; vdl=[];                % list for the sparse (order 2) tensor dl
        idq=[]; jdq=[]; kdq=[]; vdq=[];        % list for the sparse (order 3) tensor dq
                
        zi_phase=1; % index of the state variable reciving the phase condition zi'(0)=0
        % or if the system is 'conservative', it receives zi(0) = lambda,
        % while other initial conditions are free.
    end
    
    methods
        
        function sys = SystHBQ(nz,nz_aux,H,equations,point_display,global_display,parameters,subtype,writing)
            
            if nargin<8; subtype = 'autonomous'; end
            if nargin<9; writing = 'standard'; end

            sys=sys@Syst('neq',nz*(2*H+1)+1,'neq_aux',nz_aux*(2*H+1)+2);
            
            sys.type = 'HBQ';
            sys.subtype = subtype;

            sys.parameters = parameters;
            sys.equations = equations;
            sys.point_display = point_display;
            sys.global_display = global_display;
            
            sys.nz = nz;
            sys.nz_aux = nz_aux;
            sys.nz_tot = nz + nz_aux;
            sys.H  = H ;    % harmonic number
            
            % Arclength only on the main variables
            sys.arclengthdef = sparse(1:sys.neq+1,ones(sys.neq+1,1),1,sys.ninc,1);
            
            % Creation of the little operators
            switch writing
                case 'vectorial'
                    sys = sys.get_operators_vec;
                case 'standard'
                    sys = sys.get_operators;
                otherwise
                    error('SystHBQ : Uncorrect writing type of the system. It should be either standard or vectorial.');
            end
            
            % Specific treatment of the products to avoid a long loop in Fpnl.m
            [sys.uiq,sys.i_iq,~] = unique(sys.iq); % Select one product per equation to allow a vectorial allocation

            % angfreq of the oscillations for forced systems.
            if strcmp(sys.subtype,'forced')
                try sys.angfreq = sys.parameters.angfreq;
                catch; sys.angfreq = 'omega'; disp('The forcing angular frequency is the continuation parameter.');
                end
            end
            
            % Residu
            sys.R = @R;
            
            function [Rtot] = R(sys,Utot)
                %  Compute  R(U)
                H    = sys.H;   % Harmonic number
                nz_tot   = sys.nz_tot;  % number of equation of the DAE system
                DHp1 = 2*H+1 ;
                
                % Extract informations from Utot
                [Ztot,omega,lambda,omega2,lambdaomega] = sys.get_Ztot(Utot);
                Rfourier=zeros(DHp1,nz_tot);
                
                %   R = C0 + lambda C1  + L0(U) + lambda L1(U) +
                %       omega D(U) + lambdaomega D1(U) + omega2 DD(U) + Q(U,U)
                %
                %   R is formed by block using the list ic0, vc0, ......
                for i=1:numel(sys.iforce0)
                    Rfourier(sys.hforce0(i),sys.iforce0(i)) = Rfourier(sys.hforce0(i),sys.iforce0(i)) - sys.vforce0(i);
                end
                
                for i=1:numel(sys.iforce1)
                    Rfourier(sys.hforce1(i),sys.iforce1(i)) = Rfourier(sys.hforce1(i),sys.iforce1(i)) - lambda*sys.vforce1(i);
                end
                
                for i=1:numel(sys.iforce2)
                    Rfourier(sys.hforce2(i),sys.iforce2(i)) = Rfourier(sys.hforce2(i),sys.iforce2(i)) - lambda^2*sys.vforce2(i);
                end                
                
                for i=1:numel(sys.ic0)
                    Rfourier(1,sys.ic0(i))= Rfourier(1,sys.ic0(i)) + sys.vc0(i) ;
                end
                
                for i=1:numel(sys.ic1)
                    Rfourier(1,sys.ic1(i))= Rfourier(1,sys.ic1(i)) + sys.vc1(i)*lambda ;
                end
                
                for i=1:numel(sys.ic2)
                    Rfourier(1,sys.ic2(i))= Rfourier(1,sys.ic2(i)) + sys.vc2(i)*lambda^2 ;
                end
                
                for i=1:numel(sys.il0)
                    Rfourier(:,sys.il0(i))= Rfourier(:,sys.il0(i)) + sys.vl0(i)*Ztot(:,sys.jl0(i)) ;
                end
                
                for i=1:numel(sys.il1)
                    Rfourier(:,sys.il1(i))= Rfourier(:,sys.il1(i)) + (sys.vl1(i)*lambda)*Ztot(:,sys.jl1(i)) ;
                end
                
                for i=1:numel(sys.id)
                    Rfourier(:,sys.id(i))= Rfourier(:,sys.id(i)) + (sys.vd(i)*omega)*sys.D(Ztot(:,sys.jd(i))) ;
                end
                
                for i=1:numel(sys.id1)
                    Rfourier(:,sys.id1(i))= Rfourier(:,sys.id1(i)) + (sys.vd1(i)*lambdaomega)*sys.D(Ztot(:,sys.jd1(i))) ;
                end
                
                for i=1:numel(sys.idd)
                    Rfourier(:,sys.idd(i))= Rfourier(:,sys.idd(i)) + (sys.vdd(i)*omega2)*sys.D(sys.D(Ztot(:,sys.jdd(i)))) ;
                end
                
                for i=1:numel(sys.iq)
                    Rfourier(:,sys.iq(i))= Rfourier(:,sys.iq(i)) + sys.vq(i)*( sys.Prod(Ztot(:,sys.jq(i)),Ztot(:,sys.kq(i))) ) ;
                end
                
                Ztot_t0 = sum(Ztot(1:H+1,:),1);

                % phase equation
                switch sys.subtype
                    case 'autonomous'
                        Rphase= (1:H)*Ztot(H+2:2*H+1,sys.zi_phase);
                    case 'forced'
                        if isfloat(sys.angfreq)
                            Rphase = omega - sys.angfreq;
                        else
                            Rphase = omega - lambda;
                        end
                        % Essai avec traitement spcifiques des systmes conservatifs.  
                    case 'conservative'
                        Ztotp_t0 = sum((1:H)'.*Ztot(H+2:2*H+1,:),1);
                        %%% sum(ui(0)^2 + ui'(0)^2).
                        %Rphase = (Ztot_t0(sys.zi_phase)-lambda)^2 + ... 
                        %    sum(Ztot_t0((1:sys.zi_phase-1) (sys.zi_phase+1:sys.nz)).^2) + ...
                        %    sum(Ztotp_t0.^2);
                        %%% sum(ui'(0)^2).
                        Rphase = sum(Ztotp_t0.^2);
   
                end
                
                Romega2 = omega2 - omega*omega;
                Rlambdaomega = lambdaomega - lambda*omega;
                
                if ~isempty(sys.idl)
                    for i=1:numel(sys.idl)
                        Rfourier(:,sys.idl(i))= Rfourier(:,sys.idl(i)) + sys.vdl(i)*sys.D(Ztot(:,sys.jdl(i))) ;
                    end
                    
                    for i=1:numel(sys.idq)
                        Rfourier(:,sys.idq(i))= Rfourier(:,sys.idq(i)) + sys.vdq(i)*( sys.Prod(Ztot(:,sys.jdq(i)),sys.D(Ztot(:,sys.kdq(i)))) ) ;
                    end
                    
                    cond_ini = sys.equations(sys,0,[Ztot_t0 lambda]',zeros(sys.nz_tot,1),zeros(sys.nz_tot,1));
                    Rfourier(1,sys.idl) = cond_ini(sys.idl);
                end
                
                % We divide Rfourier between the main equations and the auxiliary
                % equations. We add the phase equation and the definition of omega^2 and lambda*omega
                Rfourier = reshape(Rfourier,nz_tot*DHp1,1);
                R = [Rfourier(1:sys.neq-1); Rphase];
    
                Raux = [Rfourier(sys.neq:end);Romega2;Rlambdaomega];
                Rtot = [R;Raux];
                
            end
            
            
            function [Rtot] = eq_NS_auto(sys,Utot)
                % Compute the operators needed to follow a Neimarck-Sacker bifurcation of the system
                % 'sys' given by the user.
                %
                % The original system R(U) = C + L(U) + Q(U,U) encounters a Hopf
                % bifurcation at point U0.
                % We use standard augmented systems (see Beyn - Handbook of continuation
                % method for more details) to write the augmented system
                %   R(U) = 0
                %   real(J_U (P) - 1i*kappa*P) = 0
                %   imag(J_U (P) - 1i*kappa*P) = 0
                %   |P|^2 = 1
                %   real(P).imag(P) = 0
                %
                %   Where P is an eigenvector with associated eigenvalue 1i*kappa (purely
                %   imaginary for Neimarck-Sacker bifurcation).
                %
                %   With J_U (P)= L(P) + Q(U,P) + Q(P,U) (+ dL(P) + dQ(U,P).)
                
                DHp1 = 2*H+1 ;
                
                [Ztot,omega,lambda,omega2,lambdaomega,mu,rPtot,iPtot,kappa] = get_Ztot(obj,Utot);
                
                Rfourier=zeros(DHp1,3*nz_tot);
                
                for i=1:size(sys.iforce0,1)
                    Rfourier(sys.hforce0(i),sys.iforce0(i)) = Rfourier(sys.hforce0(i),sys.iforce0(i)) - sys.vforce0(i);
                end
                
                for i=1:size(sys.iforce1,1)
                    Rfourier(sys.hforce1(i),sys.iforce1(i)) = Rfourier(sys.hforce1(i),sys.iforce1(i)) - lambda*sys.vforce1(i);
                end
                
                for i=1:size(obj.iforce2,1)
                    Rfourier(obj.hforce2(i),obj.iforce2(i)) = Rfourier(obj.hforce2(i),obj.iforce2(i)) - lambda^2*obj.vforce2(i);
                end                
                
                for i=1:size(sys.ic0,1)
                    Rfourier(1,sys.ic0(i))= Rfourier(1,sys.ic0(i)) + sys.vc0(i) ;
                end
                
                for i=1:size(sys.ic1,1)
                    Rfourier(1,sys.ic1(i))= Rfourier(1,sys.ic1(i)) + sys.vc1(i)*lambda ;
                end
                
                for i=1:size(sys.ic2,1)
                    Rfourier(1,sys.ic2(i))= Rfourier(1,sys.ic2(i)) + sys.vc2(i)*lambda^2 ;
                end
                
                for i=1:size(sys.il0,1)
                    Rfourier(:,sys.il0(i)) = Rfourier(:,sys.il0(i)) + sys.vl0(i)*Ztot(:,sys.jl0(i)) ;
                    Rfourier(:,sys.il0(i)+nz_tot) = Rfourier(:,sys.il0(i)+nz_tot) + sys.vl0(i)*rPtot(:,sys.jl0(i)) ;
                    Rfourier(:,sys.il0(i)+2*nz_tot) = Rfourier(:,sys.il0(i)+2*nz_tot) + sys.vl0(i)*iPtot(:,sys.jl0(i)) ;
                end
                
                for i=1:size(sys.il1,1)
                    Rfourier(:,sys.il1(i))= Rfourier(:,sys.il1(i)) + (sys.vl1(i)*lambda)*Ztot(:,sys.jl1(i)) ;
                    Rfourier(:,sys.il1(i)+nz_tot) = Rfourier(:,sys.il1(i)+nz_tot) + (lambda*sys.vl1(i))*rPtot(:,sys.jl1(i)) ;
                    Rfourier(:,sys.il1(i)+2*nz_tot) = Rfourier(:,sys.il1(i)+2*nz_tot) + (lambda*sys.vl1(i))*iPtot(:,sys.jl1(i)) ;
                end
                
                for i=1:size(sys.id,1)
                    Rfourier(:,sys.id(i))= Rfourier(:,sys.id(i)) + (sys.vd(i)*omega)*sys.D(Ztot(:,sys.jd(i))) ;
                    Rfourier(:,sys.id(i)+nz_tot)= Rfourier(:,sys.id(i)+nz_tot) + (sys.vd(i)*omega)*sys.D(rPtot(:,sys.jd(i))) ;
                    Rfourier(:,sys.id(i)+2*nz_tot)= Rfourier(:,sys.id(i)+2*nz_tot) + (sys.vd(i)*omega)*sys.D(iPtot(:,sys.jd(i))) ;
                end
                
                for i=1:size(sys.id1,1)
                    Rfourier(:,sys.id1(i))= Rfourier(:,sys.id1(i)) + (sys.vd1(i)*lambdaomega)*sys.D(Ztot(:,sys.jd1(i))) ;
                    Rfourier(:,sys.id1(i)+nz_tot)= Rfourier(:,sys.id1(i)+nz_tot) + (sys.vd1(i)*lambdaomega)*sys.D(rPtot(:,sys.jd1(i))) ;
                    Rfourier(:,sys.id1(i)+2*nz_tot)= Rfourier(:,sys.id1(i)+2*nz_tot) + (sys.vd1(i)*lambdaomega)*sys.D(iPtot(:,sys.jd1(i))) ;
                end
                
                for i=1:size(sys.idd,1)
                    Rfourier(:,sys.idd(i))= Rfourier(:,sys.idd(i)) + (sys.vdd(i)*omega2)*sys.D(sys.D(Ztot(:,sys.jdd(i)))) ;
                    Rfourier(:,sys.idd(i)+nz_tot)= Rfourier(:,sys.idd(i)+nz_tot) + (sys.vdd(i)*omega2)*sys.D(sys.D(rPtot(:,sys.jdd(i)))) ;
                    Rfourier(:,sys.idd(i)+2*nz_tot)= Rfourier(:,sys.idd(i)+2*nz_tot) + (sys.vdd(i)*omega2)*sys.D(sys.D(iPtot(:,sys.jdd(i)))) ;
                end
                
                for i=1:size(sys.iq,1)
                    Rfourier(:,sys.iq(i))= Rfourier(:,sys.iq(i)) + sys.vq(i)*( sys.Prod(Ztot(:,sys.jq(i)),Ztot(:,sys.kq(i))) ) ;
                    Rfourier(:,sys.iq(i)+nz_tot)= Rfourier(:,sys.iq(i)+nz_tot) + sys.vq(i)*( sys.Prod(rPtot(:,sys.jq(i)),Ztot(:,sys.kq(i))) + ...
                        sys.Prod(Ztot(:,sys.jq(i)),rPtot(:,sys.kq(i)))) ;
                    Rfourier(:,sys.iq(i)+2*nz_tot)= Rfourier(:,sys.iq(i)+2*nz_tot) + sys.vq(i)*( sys.Prod(iPtot(:,sys.jq(i)),Ztot(:,sys.kq(i))) + ...
                        sys.Prod(Ztot(:,sys.jq(i)),iPtot(:,sys.kq(i)))) ;
                end
                
                Ztot_t0 = sum(Ztot(1:H+1,:),1);

                % phase equation
                switch sys.subtype
                    case 'autonomous'
                        Rphase= (1:H)*Ztot(H+2:2*H+1,sys.zi_phase);
                    case 'forced'
                        if isfloat(sys.angfreq)
                            Rphase = omega - sys.angfreq;
                        else
                            Rphase = omega - lambda;
                        end
                    case 'conservative'
                        Ztotp_t0 = sum((1:H)'.*Ztot(H+2:2*H+1,:),1);
                        Rphase = (Ztot_t0(sys.zi_phase)-lambda)^2 + Ztotp_t0(sys.zi_phase)^2;
   
                end
                
                Romega2 = omega2 - omega*omega;
                Rlambdaomega = lambdaomega - lambda*omega;
                
                if ~isempty(sys.idl)
                    for i=1:size(sys.idl,1)
                        Rfourier(:,sys.idl(i))= Rfourier(:,sys.idl(i)) + sys.vdl(i)*sys.D(Ztot(:,sys.jdl(i))) ;
                        Rfourier(:,sys.idl(i)+nz_tot)= Rfourier(:,sys.idl(i)+nz_tot) + sys.vdl(i)*sys.D(rPtot(:,sys.jdl(i))) ;
                        Rfourier(:,sys.idl(i)+2*nz_tot)= Rfourier(:,sys.idl(i)+2*nz_tot) + sys.vdl(i)*sys.D(iPtot(:,sys.jdl(i))) ;
                    end
                    
                    for i=1:size(sys.idq,1)
                        Rfourier(:,sys.idq(i))= Rfourier(:,sys.idq(i)) + sys.vdq(i)*( sys.Prod(Ztot(:,sys.jdq(i)),sys.D(Ztot(:,sys.kdq(i)))) ) ;
                        Rfourier(:,sys.idq(i)+nz_tot)= Rfourier(:,sys.idq(i)+nz_tot) + ...
                            sys.vdq(i)*( sys.Prod(rPtot(:,sys.jdq(i)),sys.D(Ztot(:,sys.kdq(i)))) + sys.Prod(Ztot(:,sys.jdq(i)),sys.D(rPtot(:,sys.kdq(i))))) ;
                        Rfourier(:,sys.idq(i)+2*nz_tot)= Rfourier(:,sys.idq(i)+2*nz_tot) + ...
                            sys.vdq(i)*( sys.Prod(iPtot(:,sys.jdq(i)),sys.D(Ztot(:,sys.kdq(i)))) + sys.Prod(Ztot(:,sys.jdq(i)),sys.D(iPtot(:,sys.kdq(i))))) ;
                    end
                    
                    cond_ini = sys.equations(sys,0,[Ztot_t0 lambda]',zeros(sys.nz_tot,1),zeros(sys.nz_tot,1));
                    Rfourier(1,sys.idl) = cond_ini(sys.idl);
                end
                
                Rfourier(:,obj.nz_tot+1:2*obj.nz_tot) = Rfourier(:,obj.nz_tot+1:2*obj.nz_tot) + kappa*iPtot;
                Rfourier(:,2*obj.nz_tot+1:3*obj.nz_tot) = Rfourier(:,2*obj.nz_tot+1:3*obj.nz_tot) - kappa*rPtot;
                
                
                Orthogonal_eq = sum(sum(rPtot(:,1:obj.nz).*iPtot(:,1:obj.nz)));
                Normalization_eq = sum(sum(rPtot(:,1:obj.nz).^2 + iPtot(:,1:obj.nz).^2)) - 1;
                
                % We divide Rfourier between the main equations and the auxiliary
                % equations. We add the phase equation and the definition of omega^2 and lambda*omega
                Rfourier_main = reshape(Rfourier(:,1:obj.nz),obj.nz*DHp1,1);
                R = [Rfourier_main; Rphase];
                
                Rfourier_aux = reshape(Rfourier(:,obj.nz+1:obj.nz_tot),obj.nz_aux*DHp1,1);
                Rfourier_Eigen = reshape(Rfourier(:,obj.nz_tot+1:end),2*obj.nz_tot*DHp1,1);
                Raux = [Rfourier_aux;Romega2;Rlambdaomega;Rfourier_Eigen;Orthogonal_eq;Normalization_eq];
                Rtot = [R;Raux];
                
                %% How to add mu to this ?
                %%% Replace the last auxiliary equation ???!! :
                Residue_tot(end) = Uaux(end) - mu; 
                
            end
            
            
            
        end
        %% All the function used to compute the residu and the jacobian from the little operators
        function [Ztot,omega,lambda,omega2,lambdaomega,mu,rPtot,iPtot,kappa] = get_Ztot(obj,Utot)
            
            DHp1 = obj.H*2+1;
            switch obj.bif_type
                case 'nothing'
                    Z      = reshape(Utot(1:obj.neq-1),DHp1,obj.nz);
                    omega  = Utot(obj.neq);
                    lambda = Utot(obj.neq+1) ;
            
                    omega2 = Utot(end-1);
                    lambdaomega =Utot(end);
                    Zaux   = reshape(Utot(obj.neq+2:end-2),DHp1,obj.nz_aux);
            
                    Ztot = [Z Zaux];
                case 'NS'
                    op = sys.parameters.op;
                    Z      = reshape(Utot(1:op.neq-1),DHp1,obj.nz);
                    omega  = Utot(op.neq);
                    lambda = Utot(op.neq+1);
                    mu     = Utot(op.neq+2);
                    
                    Zaux  = reshape(Utot(op.neq+3:op.neq_tot+2),DHp1,obj.nz_aux);
                    omega2= Utot(op.neq_tot+3);
                    lambdaomega = Utot(op.neq_tot+4);
                    
                    rPtot = reshape(Utot(op.neq_tot+5:2*op.neq_tot+1),DHp1,obj.nz_tot);
                    iPtot = reshape(Utot(2*op.neq_tot+2:3*op.neq_tot-2),DHp1,obj.nz_tot);
                    kappa = Utot(3*op.neq_tot-1);
            
                    Ztot = [Z Zaux];
            end
        end
        
        function  DU = D(obj,U,complex)
            %  Compute  D.U
            Hu=(size(U,1)-1)/2   ;   % harmonic number
            if nargin < 3
                DU = [ zeros(1,size(U,2)) ; (1:Hu)'.*U(Hu+2:end,:) ; (-(1:Hu))'.*U(2:Hu+1,:) ];
            else
                DU = 1i*((-Hu:Hu)'.*U);
            end
        end
        
        function R = Prod(obj,U,V,complex)
            % Compute the product of U and V using complex convolution
            
            U = full(U);
            V = full(V);
            
            if nargin < 4
                Ucompl = obj.real_to_compl(U,'full');
                Vcompl = obj.real_to_compl(V,'full');
            else
                Ucompl = U;
                Vcompl = V;
            end
            
            Rcompl_tot = zeros(size(U));
            for i=1:size(U,2)
                Rcompl_tot(:,i) = conv2(Ucompl(:,i),Vcompl(:,i),'same');
            end
            Rcompl = Rcompl_tot(obj.H+1:end,:);
            R = obj.compl_to_real(Rcompl);
        end
        
        function Bmat = B(obj,U)
            %  Compute  the matrix B from U (size 2H+1 )
            %
            %  toeplitz(c,r) generate a matrix from a colum c and a row r
            %
            Hu=(size(U,1)-1)/2   ;   % harmonic number
            Uc= U(2:Hu+1)/2;           % Half of the cosines
            Us= U(Hu+2:2*Hu+1)/2;       % Half of the sines
            
            LUc= [ U(1) ; Uc(1:end-1) ];    %   [ U0 ; Uc1/2 ; Uc2/2 ...]
            LUs= [   0  ; Us(1:end-1) ];    %   [ 0  ; Us1/2 ; Us2/2 ...]
            CC = toeplitz(LUc,LUc);
            SS = toeplitz(-LUs,LUs);
            
            Cp = zeros(Hu,Hu) ;
            Sp = zeros(Hu,Hu) ;
            L0 = zeros(Hu,1) ;
            VUc= [   0  ; Uc(end:-1:2) ];     %   [ 0  ; UcH/2 ; UcH-1/2 ...]
            VUs= [   0  ; Us(end:-1:2) ];     %   [ 0  ; UsH/2 ; UsH-1/2 ...]
            Cp(:,Hu:-1:1) = toeplitz(L0,VUc);
            Sp(:,Hu:-1:1) = toeplitz(L0,VUs);
            
            Bmat(:,1)= U ;
            
            Bmat(1,2:Hu+1) = Uc ;
            Bmat(1,Hu+2:2*Hu+1) = Us ;
            
            Bmat(2:Hu+1    ,2:Hu+1    )=CC + Cp;
            Bmat(2:Hu+1    ,Hu+2:2*Hu+1)=SS + Sp;
            Bmat(Hu+2:2*Hu+1,2:Hu+1    )=SS'+ Sp;
            Bmat(Hu+2:2*Hu+1,Hu+2:2*Hu+1)=CC - Cp;
        end
        
        function Ureal = compl_to_real(obj,Ucompl)
            % function Ureal = compl_to_real(sys,Ucompl)
            % Take a vector of complex harmonics  from 0 to H and give
            % the real corresponding Fourier series (constant, cos, sin)
            Ucos = 2*real(Ucompl(2:end,:));
            Usin = -2*imag(Ucompl(2:end,:));
            Ureal = real([Ucompl(1,:); Ucos ; Usin]);
        end
        
        function Ucompl = real_to_compl(obj,Ureal,shape)
            % function Ucompl = real_to_compl(sys,Ureal)
            % It is the inverse function of compl_to_real
            Hu=(size(Ureal,1)-1)/2   ;   % harmonic number
            Uharm = 0.5*(Ureal(2:Hu+1,:)-1i*Ureal(Hu+2:end,:));
            if nargin < 3
                Ucompl = [Ureal(1,:);Uharm];
            else
                Ucompl = [flipud(conj(Uharm));Ureal(1,:);Uharm];
            end
        end
 
            
    end
end


